@@ -20,16 +20,13 @@
  *    Chema Celorio <chema@celorio.com>
  *    Lauris Kaplinski <lauris@helixcode.com>
  *
- *  Copyright 2000-2001 Ximian, Inc. and authors
+ *  Copyright 2000-2003 Ximian, Inc.
  *
  *  References:
  *    Document Structuring Conventions, Adobe. [http://partners.adobe.com/asn/developer/pdfs/tn/5001.DSC_Spec.pdf]
  */
 
-#define __GNOME_PRINT_PS2_C__
-
-#include "config.h"
-
+#include <config.h>
 #include <math.h>
 #include <string.h>
 #include <time.h>
@@ -42,10 +39,9 @@
 #include <libgnomeprint/gp-gc-private.h>
 #include <libgnomeprint/gnome-print-transport.h>
 #include <libgnomeprint/gnome-font-private.h>
-#include <libgnomeprint/gnome-print-encode-private.h>
+#include <libgnomeprint/gnome-print-encode.h>
 #include <libgnomeprint/gnome-pgl-private.h>
-
-#include "gnome-print-ps2.h"
+#include <libgnomeprint/gnome-print-ps2.h>
 
 #define EOL "\n"
 
@@ -686,7 +682,7 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	gnome_print_transport_printf (pc->transport, ">> setpagedevice" EOL);
 	/* Download fonts */
 	for (font = ps2->fonts; font != NULL; font = font->next) {
-		gnome_font_face_pso_ensure_buffer (font->pso);
+		gnome_font_face_ps_embed (font->pso);
 		gnome_print_transport_printf (pc->transport, "%%%%BeginResource: font %s" EOL, font->pso->encodedname);
 		gnome_print_transport_write (pc->transport, font->pso->buf, font->pso->length);
 		gnome_print_transport_printf (pc->transport, "%%%%EndResource" EOL);
