@@ -86,11 +86,50 @@ gpa_config_class_init (GPAConfigClass *klass)
 	node_class->verify    = gpa_config_verify;
 }
 
+/**
+ * gpa_config_printer_modified:
+ * @node: 
+ * 
+ * A different ->printer has been set, update ->settings since
+ * the old ->settings where for the previous printer
+ **/
+static void
+gpa_config_printer_modified (GPANode *node)
+{
+	GPANode *settings;
+	GPANode *printer;
+	GPAConfig *config;
+
+	g_return_if_fail (GPA_IS_REFERENCE (node));
+	g_return_if_fail (GPA_IS_CONFIG (node->parent));
+
+	config = GPA_CONFIG (node->parent);
+	printer = GPA_REFERENCE_REFERENCE (config->printer);
+	
+	if (printer == GPA_REFERENCE_REFERENCE (GPA_SETTINGS (GPA_REFERENCE_REFERENCE (config->settings))->printer)) {
+		g_print ("Nothing changed, no need to update\n");
+		return;
+	}
+
+	settings = gpa_printer_get_default_settings (GPA_PRINTER (printer));
+	gpa_reference_set_reference (GPA_REFERENCE (config->settings),
+				     settings);
+	gpa_node_emit_modified (GPA_NODE (config));
+}
+
 static void
 gpa_config_init (GPAConfig *config)
 {
-	config->printer = NULL;
-	config->settings = NULL;
+	GPANode *p, *s;
+
+	p = GPA_NODE (gpa_reference_new_emtpy ("Printer"));
+	s = GPA_NODE (gpa_reference_new_emtpy ("Settings"));
+
+	config->printer  = gpa_node_attach (GPA_NODE (config), p);
+	config->settings = gpa_node_attach (GPA_NODE (config), s);
+
+	g_signal_connect (G_OBJECT (p), "modified", (GCallback)
+			  gpa_config_printer_modified, NULL);
 }
 
 static void
@@ -113,14 +152,10 @@ gpa_config_verify (GPANode *node)
 
 	config = GPA_CONFIG (node);
 
-	if (!config->printer)
-		return FALSE;
-	if (!gpa_node_verify (config->printer))
-		return FALSE;
-	if (!config->settings)
-		return FALSE;
-	if (!gpa_node_verify (GPA_NODE (config->settings)))
-		return FALSE;
+	gpa_return_false_if_fail (config->printer);
+	gpa_return_false_if_fail (gpa_node_verify (config->printer));
+	gpa_return_false_if_fail (config->settings);
+	gpa_return_false_if_fail (gpa_node_verify (config->settings));
 
 	return TRUE;
 }
@@ -129,18 +164,14 @@ GPAConfig *
 gpa_config_new_full (GPAPrinter *printer, GPASettings * settings)
 {
 	GPAConfig * config;
-	GPANode *p, *s;
 
 	g_return_val_if_fail (GPA_IS_PRINTER (printer), NULL);
 	g_return_val_if_fail (GPA_IS_SETTINGS (settings), NULL);
 	
 	config = (GPAConfig *) gpa_node_new (GPA_TYPE_CONFIG, "GpaConfigRootNode");
 
-	p = GPA_NODE (gpa_reference_new (GPA_NODE (printer),  "Printer"));
-	s = GPA_NODE (gpa_reference_new (GPA_NODE (settings), "Settings"));
-
-	config->printer  = gpa_node_attach (GPA_NODE (config), p);
-	config->settings = gpa_node_attach (GPA_NODE (config), s);
+	gpa_reference_set_reference (GPA_REFERENCE (config->printer),  GPA_NODE (printer));
+	gpa_reference_set_reference (GPA_REFERENCE (config->settings), GPA_NODE (settings));
 	
 	gpa_node_reverse_children (GPA_NODE (config));
 
@@ -185,22 +216,6 @@ gpa_config_new_error:
 	return config;
 }
 
-void
-gpa_config_set_printer (GPAConfig *config, GPAPrinter *printer)
-{
-	GPANode *settings;
-
-	if (GPA_REFERENCE_REFERENCE (config->printer) == (GPANode *)printer)
-		return;
-
-	gpa_reference_set_reference (GPA_REFERENCE (config->printer),
-				     GPA_NODE (printer));
-	settings = gpa_printer_get_default_settings (printer);
-	gpa_reference_set_reference (GPA_REFERENCE (config->settings),
-				     GPA_NODE (settings));
-	gpa_node_emit_modified (GPA_NODE (config));
-}
-
 static GPANode *
 gpa_config_duplicate (GPANode *node)
 {
